@@ -6,7 +6,6 @@ import enum
 import functools
 import inspect
 import socket
-import sys
 import warnings
 from asyncio import AbstractEventLoopPolicy
 from textwrap import dedent
@@ -16,11 +15,14 @@ from typing import (
     Awaitable,
     Callable,
     Dict,
+    Generator,
     Iterable,
     Iterator,
     List,
     Literal,
+    Mapping,
     Optional,
+    Sequence,
     Set,
     Type,
     TypeVar,
@@ -28,14 +30,17 @@ from typing import (
     overload,
 )
 
+import pluggy
 import pytest
 from pytest import (
     Class,
     Collector,
     Config,
+    FixtureDef,
     FixtureRequest,
     Function,
     Item,
+    Mark,
     Metafunc,
     Module,
     Package,
@@ -47,24 +52,18 @@ from pytest import (
     StashKey,
 )
 
-_R = TypeVar("_R")
-
 _ScopeName = Literal["session", "package", "module", "class", "function"]
 _T = TypeVar("_T")
 
 SimpleFixtureFunction = TypeVar(
-    "SimpleFixtureFunction", bound=Callable[..., Awaitable[_R]]
+    "SimpleFixtureFunction", bound=Callable[..., Awaitable[object]]
 )
 FactoryFixtureFunction = TypeVar(
-    "FactoryFixtureFunction", bound=Callable[..., AsyncIterator[_R]]
+    "FactoryFixtureFunction", bound=Callable[..., AsyncIterator[object]]
 )
 FixtureFunction = Union[SimpleFixtureFunction, FactoryFixtureFunction]
 FixtureFunctionMarker = Callable[[FixtureFunction], FixtureFunction]
 
-# https://github.com/pytest-dev/pytest/pull/9510
-FixtureDef = Any
-SubRequest = Any
-
 
 class PytestAsyncioError(Exception):
     """Base class for exceptions raised by pytest-asyncio"""
@@ -101,6 +100,12 @@ def pytest_addoption(parser: Parser, pluginmanager: PytestPluginManager) -> None
         help="default value for --asyncio-mode",
         default="strict",
     )
+    parser.addini(
+        "asyncio_default_fixture_loop_scope",
+        type="string",
+        help="default scope of the asyncio event loop used to execute async fixtures",
+        default=None,
+    )
 
 
 @overload
@@ -108,6 +113,7 @@ def fixture(
     fixture_function: FixtureFunction,
     *,
     scope: "Union[_ScopeName, Callable[[str, Config], _ScopeName]]" = ...,
+    loop_scope: Union[_ScopeName, None] = ...,
     params: Optional[Iterable[object]] = ...,
     autouse: bool = ...,
     ids: Union[
@@ -124,6 +130,7 @@ def fixture(
     fixture_function: None = ...,
     *,
     scope: "Union[_ScopeName, Callable[[str, Config], _ScopeName]]" = ...,
+    loop_scope: Union[_ScopeName, None] = ...,
     params: Optional[Iterable[object]] = ...,
     autouse: bool = ...,
     ids: Union[
@@ -136,17 +143,19 @@ def fixture(
 
 
 def fixture(
-    fixture_function: Optional[FixtureFunction] = None, **kwargs: Any
+    fixture_function: Optional[FixtureFunction] = None,
+    loop_scope: Union[_ScopeName, None] = None,
+    **kwargs: Any,
 ) -> Union[FixtureFunction, FixtureFunctionMarker]:
     if fixture_function is not None:
-        _make_asyncio_fixture_function(fixture_function)
+        _make_asyncio_fixture_function(fixture_function, loop_scope)
         return pytest.fixture(fixture_function, **kwargs)
 
     else:
 
         @functools.wraps(fixture)
         def inner(fixture_function: FixtureFunction) -> FixtureFunction:
-            return fixture(fixture_function, **kwargs)
+            return fixture(fixture_function, loop_scope=loop_scope, **kwargs)
 
         return inner
 
@@ -156,11 +165,14 @@ def _is_asyncio_fixture_function(obj: Any) -> bool:
     return getattr(obj, "_force_asyncio_fixture", False)
 
 
-def _make_asyncio_fixture_function(obj: Any) -> None:
+def _make_asyncio_fixture_function(
+    obj: Any, loop_scope: Union[_ScopeName, None]
+) -> None:
     if hasattr(obj, "__func__"):
         # instance method, check the function object
         obj = obj.__func__
     obj._force_asyncio_fixture = True
+    obj._loop_scope = loop_scope
 
 
 def _is_coroutine_or_asyncgen(obj: Any) -> bool:
@@ -180,8 +192,20 @@ def _get_asyncio_mode(config: Config) -> Mode:
         )
 
 
+_DEFAULT_FIXTURE_LOOP_SCOPE_UNSET = """\
+The configuration option "asyncio_default_fixture_loop_scope" is unset.
+The event loop scope for asynchronous fixtures will default to the fixture caching \
+scope. Future versions of pytest-asyncio will default the loop scope for asynchronous \
+fixtures to function scope. Set the default fixture loop scope explicitly in order to \
+avoid unexpected behavior in the future. Valid fixture loop scopes are: \
+"function", "class", "module", "package", "session"
+"""
+
+
 def pytest_configure(config: Config) -> None:
-    """Inject documentation."""
+    default_loop_scope = config.getini("asyncio_default_fixture_loop_scope")
+    if not default_loop_scope:
+        warnings.warn(PytestDeprecationWarning(_DEFAULT_FIXTURE_LOOP_SCOPE_UNSET))
     config.addinivalue_line(
         "markers",
         "asyncio: "
@@ -194,7 +218,8 @@ def pytest_configure(config: Config) -> None:
 def pytest_report_header(config: Config) -> List[str]:
     """Add asyncio config to pytest header."""
     mode = _get_asyncio_mode(config)
-    return [f"asyncio: mode={mode}"]
+    default_loop_scope = config.getini("asyncio_default_fixture_loop_scope")
+    return [f"asyncio: mode={mode}, default_loop_scope={default_loop_scope}"]
 
 
 def _preprocess_async_fixtures(
@@ -202,8 +227,10 @@ def _preprocess_async_fixtures(
     processed_fixturedefs: Set[FixtureDef],
 ) -> None:
     config = collector.config
+    default_loop_scope = config.getini("asyncio_default_fixture_loop_scope")
     asyncio_mode = _get_asyncio_mode(config)
     fixturemanager = config.pluginmanager.get_plugin("funcmanage")
+    assert fixturemanager is not None
     for fixtures in fixturemanager._arg2fixturedefs.values():
         for fixturedef in fixtures:
             func = fixturedef.func
@@ -215,15 +242,14 @@ def _preprocess_async_fixtures(
                 # Ignore async fixtures without explicit asyncio mark in strict mode
                 # This applies to pytest_trio fixtures, for example
                 continue
-            scope = fixturedef.scope
-            if scope == "function":
-                event_loop_fixture_id = "event_loop"
-            else:
-                event_loop_node = _retrieve_scope_root(collector, scope)
-                event_loop_fixture_id = event_loop_node.stash.get(
-                    _event_loop_fixture_id, None
-                )
-            _make_asyncio_fixture_function(func)
+            scope = (
+                getattr(func, "_loop_scope", None)
+                or default_loop_scope
+                or fixturedef.scope
+            )
+            if scope == "function" and "event_loop" not in fixturedef.argnames:
+                fixturedef.argnames += ("event_loop",)
+            _make_asyncio_fixture_function(func, scope)
             function_signature = inspect.signature(func)
             if "event_loop" in function_signature.parameters:
                 warnings.warn(
@@ -234,57 +260,39 @@ def _preprocess_async_fixtures(
                         f"instead."
                     )
                 )
-            _inject_fixture_argnames(fixturedef, event_loop_fixture_id)
-            _synchronize_async_fixture(fixturedef, event_loop_fixture_id)
+            if "request" not in fixturedef.argnames:
+                fixturedef.argnames += ("request",)
+            _synchronize_async_fixture(fixturedef)
             assert _is_asyncio_fixture_function(fixturedef.func)
             processed_fixturedefs.add(fixturedef)
 
 
-def _inject_fixture_argnames(
-    fixturedef: FixtureDef, event_loop_fixture_id: str
-) -> None:
-    """
-    Ensures that `request` and `event_loop` are arguments of the specified fixture.
-    """
-    to_add = []
-    for name in ("request", event_loop_fixture_id):
-        if name not in fixturedef.argnames:
-            to_add.append(name)
-    if to_add:
-        fixturedef.argnames += tuple(to_add)
-
-
-def _synchronize_async_fixture(
-    fixturedef: FixtureDef, event_loop_fixture_id: str
-) -> None:
+def _synchronize_async_fixture(fixturedef: FixtureDef) -> None:
     """
     Wraps the fixture function of an async fixture in a synchronous function.
     """
     if inspect.isasyncgenfunction(fixturedef.func):
-        _wrap_asyncgen_fixture(fixturedef, event_loop_fixture_id)
+        _wrap_asyncgen_fixture(fixturedef)
     elif inspect.iscoroutinefunction(fixturedef.func):
-        _wrap_async_fixture(fixturedef, event_loop_fixture_id)
+        _wrap_async_fixture(fixturedef)
 
 
 def _add_kwargs(
     func: Callable[..., Any],
     kwargs: Dict[str, Any],
-    event_loop_fixture_id: str,
     event_loop: asyncio.AbstractEventLoop,
-    request: SubRequest,
+    request: FixtureRequest,
 ) -> Dict[str, Any]:
     sig = inspect.signature(func)
     ret = kwargs.copy()
     if "request" in sig.parameters:
         ret["request"] = request
-    if event_loop_fixture_id in sig.parameters:
-        ret[event_loop_fixture_id] = event_loop
+    if "event_loop" in sig.parameters:
+        ret["event_loop"] = event_loop
     return ret
 
 
-def _perhaps_rebind_fixture_func(
-    func: _T, instance: Optional[Any], unittest: bool
-) -> _T:
+def _perhaps_rebind_fixture_func(func: _T, instance: Optional[Any]) -> _T:
     if instance is not None:
         # The fixture needs to be bound to the actual request.instance
         # so it is bound to the same object as the test method.
@@ -293,29 +301,28 @@ def _perhaps_rebind_fixture_func(
             unbound, cls = func.__func__, type(func.__self__)  # type: ignore
         except AttributeError:
             pass
-        # If unittest is true, the fixture is bound unconditionally.
-        # otherwise, only if the fixture was bound before to an instance of
+        # Only if the fixture was bound before to an instance of
         # the same type.
-        if unittest or (cls is not None and isinstance(instance, cls)):
+        if cls is not None and isinstance(instance, cls):
             func = unbound.__get__(instance)  # type: ignore
     return func
 
 
-def _wrap_asyncgen_fixture(fixturedef: FixtureDef, event_loop_fixture_id: str) -> None:
+def _wrap_asyncgen_fixture(fixturedef: FixtureDef) -> None:
     fixture = fixturedef.func
 
     @functools.wraps(fixture)
-    def _asyncgen_fixture_wrapper(request: SubRequest, **kwargs: Any):
-        func = _perhaps_rebind_fixture_func(
-            fixture, request.instance, fixturedef.unittest
-        )
-        event_loop = kwargs.pop(event_loop_fixture_id)
-        gen_obj = func(
-            **_add_kwargs(func, kwargs, event_loop_fixture_id, event_loop, request)
+    def _asyncgen_fixture_wrapper(request: FixtureRequest, **kwargs: Any):
+        func = _perhaps_rebind_fixture_func(fixture, request.instance)
+        event_loop_fixture_id = _get_event_loop_fixture_id_for_async_fixture(
+            request, func
         )
+        event_loop = request.getfixturevalue(event_loop_fixture_id)
+        kwargs.pop(event_loop_fixture_id, None)
+        gen_obj = func(**_add_kwargs(func, kwargs, event_loop, request))
 
         async def setup():
-            res = await gen_obj.__anext__()
+            res = await gen_obj.__anext__()  # type: ignore[union-attr]
             return res
 
         def finalizer() -> None:
@@ -323,7 +330,7 @@ def _wrap_asyncgen_fixture(fixturedef: FixtureDef, event_loop_fixture_id: str) -
 
             async def async_finalizer() -> None:
                 try:
-                    await gen_obj.__anext__()
+                    await gen_obj.__anext__()  # type: ignore[union-attr]
                 except StopAsyncIteration:
                     pass
                 else:
@@ -337,28 +344,48 @@ def _wrap_asyncgen_fixture(fixturedef: FixtureDef, event_loop_fixture_id: str) -
         request.addfinalizer(finalizer)
         return result
 
-    fixturedef.func = _asyncgen_fixture_wrapper
+    fixturedef.func = _asyncgen_fixture_wrapper  # type: ignore[misc]
 
 
-def _wrap_async_fixture(fixturedef: FixtureDef, event_loop_fixture_id: str) -> None:
+def _wrap_async_fixture(fixturedef: FixtureDef) -> None:
     fixture = fixturedef.func
 
     @functools.wraps(fixture)
-    def _async_fixture_wrapper(request: SubRequest, **kwargs: Any):
-        func = _perhaps_rebind_fixture_func(
-            fixture, request.instance, fixturedef.unittest
+    def _async_fixture_wrapper(request: FixtureRequest, **kwargs: Any):
+        func = _perhaps_rebind_fixture_func(fixture, request.instance)
+        event_loop_fixture_id = _get_event_loop_fixture_id_for_async_fixture(
+            request, func
         )
-        event_loop = kwargs.pop(event_loop_fixture_id)
+        event_loop = request.getfixturevalue(event_loop_fixture_id)
+        kwargs.pop(event_loop_fixture_id, None)
 
         async def setup():
-            res = await func(
-                **_add_kwargs(func, kwargs, event_loop_fixture_id, event_loop, request)
-            )
+            res = await func(**_add_kwargs(func, kwargs, event_loop, request))
             return res
 
         return event_loop.run_until_complete(setup())
 
-    fixturedef.func = _async_fixture_wrapper
+    fixturedef.func = _async_fixture_wrapper  # type: ignore[misc]
+
+
+def _get_event_loop_fixture_id_for_async_fixture(
+    request: FixtureRequest, func: Any
+) -> str:
+    default_loop_scope = request.config.getini("asyncio_default_fixture_loop_scope")
+    loop_scope = (
+        getattr(func, "_loop_scope", None) or default_loop_scope or request.scope
+    )
+    if loop_scope == "function":
+        event_loop_fixture_id = "event_loop"
+    else:
+        event_loop_node = _retrieve_scope_root(request._pyfuncitem, loop_scope)
+        event_loop_fixture_id = event_loop_node.stash.get(
+            # Type ignored because of non-optimal mypy inference.
+            _event_loop_fixture_id,  # type: ignore[arg-type]
+            "",
+        )
+    assert event_loop_fixture_id
+    return event_loop_fixture_id
 
 
 class PytestAsyncioFunction(Function):
@@ -395,7 +422,8 @@ class PytestAsyncioFunction(Function):
             keywords=function.keywords,
             originalname=function.originalname,
         )
-        subclass_instance.own_markers.extend(function.own_markers)
+        subclass_instance.own_markers = function.own_markers
+        assert subclass_instance.own_markers == function.own_markers
         subclassed_function_signature = inspect.signature(subclass_instance.obj)
         if "event_loop" in subclassed_function_signature.parameters:
             subclass_instance.warn(
@@ -481,9 +509,11 @@ class AsyncHypothesisTest(PytestAsyncioFunction):
     @staticmethod
     def _can_substitute(item: Function) -> bool:
         func = item.obj
-        return getattr(
-            func, "is_hypothesis_test", False
-        ) and asyncio.iscoroutinefunction(func.hypothesis.inner_test)
+        return (
+            getattr(func, "is_hypothesis_test", False)  # type: ignore[return-value]
+            and getattr(func, "hypothesis", None)
+            and asyncio.iscoroutinefunction(func.hypothesis.inner_test)
+        )
 
     def runtest(self) -> None:
         self.obj.hypothesis.inner_test = wrap_in_sync(
@@ -515,20 +545,27 @@ def pytest_pycollect_makeitem_preprocess_async_fixtures(
 @pytest.hookimpl(specname="pytest_pycollect_makeitem", hookwrapper=True)
 def pytest_pycollect_makeitem_convert_async_functions_to_subclass(
     collector: Union[pytest.Module, pytest.Class], name: str, obj: object
-) -> Union[
-    pytest.Item, pytest.Collector, List[Union[pytest.Item, pytest.Collector]], None
-]:
+) -> Generator[None, pluggy.Result, None]:
     """
     Converts coroutines and async generators collected as pytest.Functions
     to AsyncFunction items.
     """
     hook_result = yield
-    node_or_list_of_nodes = hook_result.get_result()
+    try:
+        node_or_list_of_nodes: Union[
+            pytest.Item,
+            pytest.Collector,
+            List[Union[pytest.Item, pytest.Collector]],
+            None,
+        ] = hook_result.get_result()
+    except BaseException as e:
+        hook_result.force_exception(e)
+        return
     if not node_or_list_of_nodes:
         return
-    try:
+    if isinstance(node_or_list_of_nodes, Sequence):
         node_iterator = iter(node_or_list_of_nodes)
-    except TypeError:
+    else:
         # Treat single node as a single-element iterable
         node_iterator = iter((node_or_list_of_nodes,))
     updated_node_collection = []
@@ -547,8 +584,8 @@ def pytest_pycollect_makeitem_convert_async_functions_to_subclass(
     hook_result.force_result(updated_node_collection)
 
 
-_event_loop_fixture_id = StashKey[str]
-_fixture_scope_by_collector_type = {
+_event_loop_fixture_id = StashKey[str]()
+_fixture_scope_by_collector_type: Mapping[Type[pytest.Collector], _ScopeName] = {
     Class: "class",
     # Package is a subclass of module and the dict is used in isinstance checks
     # Therefore, the order matters and Package needs to appear before Module
@@ -563,7 +600,7 @@ __package_loop_stack: List[Union[FixtureFunctionMarker, FixtureFunction]] = []
 
 
 @pytest.hookimpl
-def pytest_collectstart(collector: pytest.Collector):
+def pytest_collectstart(collector: pytest.Collector) -> None:
     try:
         collector_scope = next(
             scope
@@ -637,23 +674,17 @@ def pytest_collectstart(collector: pytest.Collector):
                     pass
             return collector.__original_collect()
 
-        collector.__original_collect = collector.collect
-        collector.collect = _patched_collect
+        collector.__original_collect = collector.collect  # type: ignore[attr-defined]
+        collector.collect = _patched_collect  # type: ignore[method-assign]
     elif isinstance(collector, Class):
         collector.obj.__pytest_asyncio_scoped_event_loop = scoped_event_loop
 
 
-def _removesuffix(s: str, suffix: str) -> str:
-    if sys.version_info < (3, 9):
-        return s[: -len(suffix)]
-    return s.removesuffix(suffix)
-
-
 @contextlib.contextmanager
 def _temporary_event_loop_policy(policy: AbstractEventLoopPolicy) -> Iterator[None]:
     old_loop_policy = asyncio.get_event_loop_policy()
     try:
-        old_loop = asyncio.get_event_loop()
+        old_loop = _get_event_loop_no_warn()
     except RuntimeError:
         old_loop = None
     asyncio.set_event_loop_policy(policy)
@@ -667,7 +698,7 @@ def _temporary_event_loop_policy(policy: AbstractEventLoopPolicy) -> Iterator[No
         # subsequent tests from side-effects. We close this loop before restoring
         # the old loop to avoid ResourceWarnings.
         try:
-            asyncio.get_event_loop().close()
+            _get_event_loop_no_warn().close()
         except RuntimeError:
             pass
         asyncio.set_event_loop(old_loop)
@@ -692,7 +723,7 @@ def pytest_generate_tests(metafunc: Metafunc) -> None:
     marker = metafunc.definition.get_closest_marker("asyncio")
     if not marker:
         return
-    scope = marker.kwargs.get("scope", "function")
+    scope = _get_marked_loop_scope(marker)
     if scope == "function":
         return
     event_loop_node = _retrieve_scope_root(metafunc.definition, scope)
@@ -706,6 +737,7 @@ def pytest_generate_tests(metafunc: Metafunc) -> None:
         if event_loop_fixture_id in metafunc.fixturenames:
             return
         fixturemanager = metafunc.config.pluginmanager.get_plugin("funcmanage")
+        assert fixturemanager is not None
         if "event_loop" in metafunc.fixturenames:
             raise MultipleEventLoopsRequestedError(
                 _MULTIPLE_LOOPS_REQUESTED_ERROR.format(
@@ -726,8 +758,8 @@ def pytest_generate_tests(metafunc: Metafunc) -> None:
 
 @pytest.hookimpl(hookwrapper=True)
 def pytest_fixture_setup(
-    fixturedef: FixtureDef, request: SubRequest
-) -> Optional[object]:
+    fixturedef: FixtureDef,
+) -> Generator[None, pluggy.Result, None]:
     """Adjust the event loop policy when an event loop is produced."""
     if fixturedef.argname == "event_loop":
         # The use of a fixture finalizer is preferred over the
@@ -742,7 +774,7 @@ def pytest_fixture_setup(
             _provide_clean_event_loop,
         )
         outcome = yield
-        loop = outcome.get_result()
+        loop: asyncio.AbstractEventLoop = outcome.get_result()
         # Weird behavior was observed when checking for an attribute of FixtureDef.func
         # Instead, we now check for a special attribute of the returned event loop
         fixture_filename = inspect.getsourcefile(fixturedef.func)
@@ -755,9 +787,7 @@ def pytest_fixture_setup(
             )
         policy = asyncio.get_event_loop_policy()
         try:
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore", DeprecationWarning)
-                old_loop = policy.get_event_loop()
+            old_loop = _get_event_loop_no_warn(policy)
             is_pytest_asyncio_loop = getattr(old_loop, "__pytest_asyncio", False)
             if old_loop is not loop and not is_pytest_asyncio_loop:
                 old_loop.close()
@@ -819,9 +849,7 @@ def _restore_event_loop_policy(previous_policy) -> Callable[[], None]:
         # Close any event loop associated with the old loop policy
         # to avoid ResourceWarnings in the _provide_clean_event_loop finalizer
         try:
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore", DeprecationWarning)
-                loop = previous_policy.get_event_loop()
+            loop = _get_event_loop_no_warn(previous_policy)
         except RuntimeError:
             loop = None
         if loop:
@@ -843,6 +871,17 @@ def _provide_clean_event_loop() -> None:
     policy.set_event_loop(new_loop)
 
 
+def _get_event_loop_no_warn(
+    policy: Optional[AbstractEventLoopPolicy] = None,
+) -> asyncio.AbstractEventLoop:
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", DeprecationWarning)
+        if policy is not None:
+            return policy.get_event_loop()
+        else:
+            return asyncio.get_event_loop()
+
+
 @pytest.hookimpl(tryfirst=True, hookwrapper=True)
 def pytest_pyfunc_call(pyfuncitem: Function) -> Optional[object]:
     """
@@ -865,6 +904,7 @@ def pytest_pyfunc_call(pyfuncitem: Function) -> Optional[object]:
                 )
             )
     yield
+    return None
 
 
 def wrap_in_sync(
@@ -883,7 +923,7 @@ def wrap_in_sync(
     @functools.wraps(func)
     def inner(*args, **kwargs):
         coro = func(*args, **kwargs)
-        _loop = asyncio.get_event_loop()
+        _loop = _get_event_loop_no_warn()
         task = asyncio.ensure_future(coro, loop=_loop)
         try:
             _loop.run_until_complete(task)
@@ -915,7 +955,7 @@ def pytest_runtest_setup(item: pytest.Item) -> None:
     marker = item.get_closest_marker("asyncio")
     if marker is None:
         return
-    scope = marker.kwargs.get("scope", "function")
+    scope = _get_marked_loop_scope(marker)
     if scope != "function":
         parent_node = _retrieve_scope_root(item, scope)
         event_loop_fixture_id = parent_node.stash[_event_loop_fixture_id]
@@ -929,11 +969,39 @@ def pytest_runtest_setup(item: pytest.Item) -> None:
         obj, "is_hypothesis_test", False
     ):
         pytest.fail(
-            "test function `%r` is using Hypothesis, but pytest-asyncio "
-            "only works with Hypothesis 3.64.0 or later." % item
+            f"test function `{item!r}` is using Hypothesis, but pytest-asyncio "
+            "only works with Hypothesis 3.64.0 or later."
         )
 
 
+_DUPLICATE_LOOP_SCOPE_DEFINITION_ERROR = """\
+An asyncio pytest marker defines both "scope" and "loop_scope", \
+but it should only use "loop_scope".
+"""
+
+_MARKER_SCOPE_KWARG_DEPRECATION_WARNING = """\
+The "scope" keyword argument to the asyncio marker has been deprecated. \
+Please use the "loop_scope" argument instead.
+"""
+
+
+def _get_marked_loop_scope(asyncio_marker: Mark) -> _ScopeName:
+    assert asyncio_marker.name == "asyncio"
+    if asyncio_marker.args or (
+        asyncio_marker.kwargs and set(asyncio_marker.kwargs) - {"loop_scope", "scope"}
+    ):
+        raise ValueError("mark.asyncio accepts only a keyword argument 'scope'.")
+    if "scope" in asyncio_marker.kwargs:
+        if "loop_scope" in asyncio_marker.kwargs:
+            raise pytest.UsageError(_DUPLICATE_LOOP_SCOPE_DEFINITION_ERROR)
+        warnings.warn(PytestDeprecationWarning(_MARKER_SCOPE_KWARG_DEPRECATION_WARNING))
+    scope = asyncio_marker.kwargs.get("loop_scope") or asyncio_marker.kwargs.get(
+        "scope", "function"
+    )
+    assert scope in {"function", "class", "module", "package", "session"}
+    return scope
+
+
 def _retrieve_scope_root(item: Union[Collector, Item], scope: str) -> Collector:
     node_type_by_scope = {
         "class": Class,
@@ -944,6 +1012,7 @@ def _retrieve_scope_root(item: Union[Collector, Item], scope: str) -> Collector:
     scope_root_type = node_type_by_scope[scope]
     for node in reversed(item.listchain()):
         if isinstance(node, scope_root_type):
+            assert isinstance(node, pytest.Collector)
             return node
     error_message = (
         f"{item.name} is marked to be run in an event loop with scope {scope}, "
